// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"
#include <vector>
#include <SparseArray.h>


// [TODO] template DistanceCalculator on ID type  (and then template this class)


namespace rms {

class DistanceCache
{
public:
	static float INVALID_VALUE;

	DistanceCache(bool bUseSparseArray = false);
	~DistanceCache(void);

	void Resize(unsigned int nSize);

	float GetValue( unsigned int i1, unsigned int i2 );
	void SetValue( unsigned int i1, unsigned int i2, float fValue );

	//! clears current values
	void SetUseSparse(bool bEnable);

	class DistanceCalculator {
		public:
		virtual float GetDistance( unsigned int i1, unsigned int i2 ) = 0;
	};
	void SetCalculator( DistanceCalculator * pCalc );

protected:
	unsigned int m_nSize;
	
	bool m_bUseSparseArray;
	DynamicVector<float> m_vDistances;
	//std::vector<float> m_vDistances;
	SparseArray<float> m_vSparseDistances;

	DistanceCalculator * m_pCalculator;
};



}   // end namespace rms