// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"
#include <vector>
#include <VFTriangleMesh.h>
#include <Frame.h>

namespace rms {

class IMeshDeformer
{
public:
	IMeshDeformer() { 
		m_fGlobalScale = 1.0f;
	}

	virtual void SetMesh(rms::VFTriangleMesh * pMesh) = 0;
	
	virtual void AddBoundaryConstraints(float fWeight) = 0;
	virtual void ClearConstraints() = 0;
	virtual void UpdatePositionConstraint( IMesh::VertexID vID, const Wml::Vector3f & vPosition, float fWeight ) = 0;
	virtual void UpdateOrientationConstraint( IMesh::VertexID vID, const rms::Frame3f & vFrame, float fWeight ) = 0;

	virtual float GlobalScale() const { return m_fGlobalScale; }
	virtual void SetGlobalScale(float fGlobalScale) { m_fGlobalScale = fGlobalScale; }

	virtual rms::Frame3f GetCurrentFrame( IMesh::VertexID vID ) = 0;

	virtual void Solve() = 0;

	virtual void DebugRender() {}

protected:
	float m_fGlobalScale;
};



}   // end namespace rms