// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"

#include <Wm4Vector3.h>
#include <vector>

namespace rms {

/*
 * currently assumes T is something we can use to index into a vector (hence MaxID() function)
 */
template <class T>
class INeighbourSource
{
public:
	virtual void GetNeighbours( T nID, std::vector<T> & vNbrIDs ) = 0;
	virtual T MaxID() = 0;

	virtual bool IsOrdered() { return false; }
};


/*
 * currently assumes T is something we can use to index into a vector (hence MaxID() function)
 */
template<class T>
class IPositionSource {
public:
	virtual void GetPosition( T nID, Wml::Vector3f & vPosition, Wml::Vector3f * pNormal = NULL ) = 0;
	virtual T MaxID() = 0;
};



} // end namespace rms