// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"
#include <Frame.h>
#include <Wm4Matrix4.h>
#include <Wm4Quaternion.h>

namespace rms {

class MatrixBlender
{
public:
	enum CombinerMode {
		AverageMatrix,
		AverageQuaternion,
		AverageSO3,
		AverageMatrixExp
	};

	MatrixBlender(CombinerMode eMode) { m_eMode = eMode; }
	~MatrixBlender(void) {}

	CombinerMode & Mode() { return m_eMode; }
	const CombinerMode & Mode() const { return m_eMode; } 

	void Reset();

	void AppendMatrix( float fWeight, const Wml::Matrix3f & vMatrix );

	Wml::Matrix3f Result();

protected:
	CombinerMode m_eMode;
	
	float m_fWeightSum;

	Wml::Matrix3f m_vAvgMatrix;
	Wml::Matrix4f m_vAvgExpMatrix;

	Wml::Quaternionf m_vOrientationQuaternion;
	Wml::Quaternionf m_vAvgQuaternion;
	Wml::Vector3f m_vAvgSO3;

	static Wml::Quaternionf Mat2Quat( const Wml::Matrix3f & mat );
	static Wml::Matrix3f Quat2Mat( const Wml::Quaternionf & quat );

};



}   // end namespace rms