// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"
#include <vector>
#include <VFTriangleMesh.h>


namespace rms {

class MeshCurvature
{
public:
	MeshCurvature();

	enum MeanCurvatureMode {
		MeanCurvature_Normal,				// calls MeanCurvature_NormalSK01()			
	};

	/*
	 * generic interface, stores mean-curvature values internally, uses caches when possible
	 */

	void MeanCurvature( MeanCurvatureMode eMode, VFTriangleMesh * pMesh, const std::vector<IMesh::VertexID> & vSelection );
	float MeanCurvature( MeanCurvatureMode eMode, VFTriangleMesh * pMesh, IMesh::VertexID vID);



	/*
	 * direct computation (no caching or storage)
	 */

	//! Schneider & Kobbelt 01 version of Moreton & Sequin 92
	float MeanCurvature_NormalSK01( VFTriangleMesh * pMesh, IMesh::VertexID vID );


protected:
	rms::VFTriangleMesh * m_pMesh;

	std::vector<float> m_vH;
};



}   // end namespace rms