// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"
#include <vector>
#include <VFTriangleMesh.h>


// predecl to avoid include
namespace gsi {
	class SparseSymmetricMatrixSolver;
};


namespace rms {



class MeshVFunctionf
{
public:
	VFTriangleMesh * m_pMesh;
	IVertexFunctionf * V;

	MeshVFunctionf(VFTriangleMesh * pMesh, IVertexFunctionf * pFunction);

	void ClearConstraints();
	void ConstrainValue( IMesh::VertexID vID, float * pValue, unsigned int nValueSize = 1 );

	//! interpolates boundary values to interior. Can also constrain interior vertices.
	bool SolveDirichlet();

protected:

	struct Constraint {
		IMesh::VertexID vID;
		std::vector<float> vValue;
		bool operator<(const Constraint & c2 ) const 
			{ return vID < c2.vID; }
		Constraint() 
			{ vID = IMesh::InvalidID; }
		Constraint(IMesh::VertexID id) 
			{ vID = id; }
	};
	std::set<Constraint> m_vConstraints;

	gsi::SparseSymmetricMatrixSolver * m_pSolver;
	gsi::SparseSymmetricMatrixSolver * Solver();
};



}   // end namespace rms