// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"
#include <VFTriangleMesh.h>
#include <ExpMapGenerator.h>

class MeshGeodesic
{
public:
	MeshGeodesic( void );
	MeshGeodesic( const MeshGeodesic & m2 );
	~MeshGeodesic(void);

	void SetSurface(rms::VFTriangleMesh * pMesh, rms::IMeshBVTree * pBVTree, rms::ExpMapGenerator * pExpgen);

	void SetPoint1( const rms::Frame3f & vPoint1 )
		{ m_vPoint1 = vPoint1; Invalidate(); }
	void SetPoint2( const rms::Frame3f & vPoint2 )
		{ m_vPoint2 = vPoint2; Invalidate(); }

	void SetSurfacePoints( const std::vector<rms::Frame3f> & vCurve );

	void Clear();

	void DoOptimizeStep() { OptimizePath(); }

	void Transport( const Wml::Vector2f & vDirection, const rms::Frame3f * pPrevFrame = NULL );	
	void GetEndpoints( rms::Frame3f & vEndPoint1, rms::Frame3f & vEndPoint2 );

	void Render(const Wml::ColorRGBA & cEdgeColor, bool bDrawDijkstraPath = true, bool bDrawEndpoints = true);

	float Distance( const Wml::Vector3f & vVertex, Wml::Vector3f & vNearest );

	const std::vector<rms::Frame3f> & GetPath();

protected:
	rms::VFTriangleMesh * m_pMesh;
	rms::IMeshBVTree * m_pBVTree;
	rms::ExpMapGenerator * m_pExpGen;

	bool m_bIsValid;
	void Validate();
	void Invalidate() { m_bIsValid = false; m_vDijkstraPath.resize(0); }

	rms::Frame3f m_vPoint1;
	rms::Frame3f m_vPoint2;

	std::vector< rms::Frame3f > m_vDijkstraPath;

	void OptimizePath();
	std::vector< rms::Frame3f > m_vOptPath;
	std::vector< Wml::Vector2f > m_vLocalUVs;
};
