// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once
#include "config.h"
#include <VFTriangleMesh.h>
#include <MeshPolygons.h>
#include <GSurface.h>
#include <string>

namespace rms {

class MeshIO
{
public:
	enum MeshFormats {
		Format_OBJ,
		Format_OFF,
		Format_STL,
		Format_COLLADA,
		Format_Unknown
	};

	MeshIO(const char * pFilename, VFTriangleMesh *pMesh, MeshPolygons *pPolygonSets = NULL );
	MeshIO(const char * pFilename, GSurface * pSurface );
	MeshIO(const char * pFilename, const GSurface * pSurface );

	//! if this flag is set, we write the GSurface/MeshPolygons uvs, instead of the VFTriangleMesh UVSet
	void SetWritePerPolygonUVs( bool bEnable ) { m_bWritePerPolygonUVs = bEnable; }
	bool GetWritePerPolygonUVs( ) { return m_bWritePerPolygonUVs; }
	

	MeshFormats Format() const { return m_eFormat; }

	bool Read();
	bool Write();

	const std::string & Filename() const { return m_filename; }
	const std::string & GetLastError() const { return m_errstring; }

protected:
	std::string m_filename;
	std::string m_filenameNoSuffix;
	std::string m_errstring;

	MeshFormats m_eFormat;
	void DetermineFormat();

	bool m_bWritePerPolygonUVs;

	GSurface * m_pSurface;
	VFTriangleMesh * m_pMesh;
	MeshPolygons * m_pPolygonSets;

	const GSurface * m_pWriteOnlySurface;
	const VFTriangleMesh * m_pWriteOnlyMesh;
	const MeshPolygons * m_pWriteOnlyPolygons;

	bool Read_OBJ();
	bool Write_OBJ();

	bool Read_OFF();

	bool Write_STL();

	bool Write_COLLADA();
};

}  // end namespace rms
