// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"
#include <vector>
#include <VFTriangleMesh.h>


namespace rms {


/*
 * Compute vertex laplacians for multiple orders. Computation is done on demand.
 */
class MeshLaplacian
{
public:
	struct VertexLaplacian {
		int nOrder;
		float fWi;
		std::vector<IMesh::VertexID> vNbrs;
		std::vector<float> vWij;
		float fWii;
	};
	enum WeightType {
		UniformWeight,
		CotangentWeight
	};

	MeshLaplacian( VFTriangleMesh * pMesh, WeightType eWeightType );

	VertexLaplacian & Get(IMesh::VertexID vID, unsigned int nOrder = 1);
	const VertexLaplacian & Get(IMesh::VertexID vID, unsigned int nOrder = 1) const;


protected:
	VFTriangleMesh * m_pMesh;
	WeightType m_eWeightType;

	struct VertexSet {
		std::vector<VertexLaplacian> vOrders;
	};
	std::vector<VertexSet> m_vVertices;

	void ComputeLaplacian( IMesh::VertexID vID, unsigned int nOrder );
};




}   // end namespace rms