// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include <VFTriangleMesh.h>
#include "DijkstraFrontProp.h"


namespace rms {


class FacePositionSource : public IPositionSource<IMesh::TriangleID>
{
public:
	std::vector<Wml::Vector3f> * pCenters;
	FacePositionSource(std::vector<Wml::Vector3f> * centers)
		{ pCenters = centers; }

	virtual void GetPosition( IMesh::TriangleID nID, Wml::Vector3f & vPosition, Wml::Vector3f * pNormal ) {
		vPosition = (*pCenters)[nID];
	}
	virtual IMesh::TriangleID MaxID() {
		return (IMesh::TriangleID)pCenters->size();
	}
};

class TriangleNeighbourSource : public INeighbourSource<IMesh::TriangleID>
{
public:
	VFTriangleMesh * pMesh;
	TriangleNeighbourSource( VFTriangleMesh * mesh ) 
		{ pMesh = mesh; }

	virtual void GetNeighbours( IMesh::TriangleID nID, std::vector<IMesh::TriangleID> & vNbrIDs )
	{ IMesh::TriangleID tNbrs[3];
	  pMesh->FindNeighbours(nID, tNbrs);
	  for ( int j = 0; j < 3; ++j )
		  if ( tNbrs[j] != IMesh::InvalidID )
			  vNbrIDs.push_back(tNbrs[j]);
	}
	virtual IMesh::TriangleID MaxID() 
		{ return pMesh->GetMaxTriangleID(); }
};




class VertexPositionSource : public IPositionSource<IMesh::TriangleID>
{
public:
	VFTriangleMesh * pMesh;
	VertexPositionSource(VFTriangleMesh * mesh)
		{ pMesh = mesh; }

	virtual void GetPosition( IMesh::VertexID nID, Wml::Vector3f & vPosition, Wml::Vector3f * pNormal ) {
		pMesh->GetVertex(nID, vPosition, pNormal);
	}
	virtual IMesh::VertexID MaxID() {
		return pMesh->GetMaxVertexID();
	}
};

class VertexOneRingNeighbourSource : public INeighbourSource<IMesh::TriangleID>
{
public:
	VFTriangleMesh * pMesh;
	VertexOneRingNeighbourSource( VFTriangleMesh * mesh ) 
		{ pMesh = mesh; }

	virtual void GetNeighbours( IMesh::VertexID nID, std::vector<IMesh::VertexID> & vNbrIDs ) { 
		pMesh->VertexOneRing(nID, vNbrIDs, false);
	}
	virtual IMesh::VertexID MaxID() {
		return pMesh->GetMaxVertexID();
	}
};


}  // end namespace rms