// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"
#include "IGeometry.h"
#include <vector>


namespace rms {


template <class T>
class NeighbourCache : public INeighbourSource<T>
{
public:
	NeighbourCache(void) { m_pSource = NULL; }
	~NeighbourCache(void) { m_pSource = NULL; }

	void SetSource( INeighbourSource<T> * pSource ) { 
		m_pSource = pSource; 
		m_nSize = m_pSource->MaxID();
		m_vNeighbours.resize(0);
		m_vNeighbours.resize(m_nSize);
		m_vInitialized.resize(0);
		m_vInitialized.resize(m_nSize, false);
		nComputed = 0;
	}

	struct NeighbourSet {
		std::vector<T> vNbrs;
	};

	const NeighbourSet & GetNeighbours( T nID ) {
		NeighbourSet & s = m_vNeighbours[nID];
		if ( ! m_vInitialized[nID] ) {
			m_pSource->GetNeighbours(nID, s.vNbrs);
			m_vInitialized[nID] = true;
			++nComputed;
		}
		return s;
	}


	/* 
	 * INeighbourSource interface
	 */
	virtual void GetNeighbours( T nID, std::vector<T> & vNbrIDs ) 
		{ vNbrIDs = GetNeighbours(nID).vNbrs; }
	virtual T MaxID() { return m_pSource->MaxID(); }
	virtual bool IsOrdered() { return m_pSource->IsOrdered(); }


protected:
	INeighbourSource<T> * m_pSource;
	T m_nSize;
	std::vector<NeighbourSet> m_vNeighbours;
	std::vector<bool> m_vInitialized;

	unsigned int nComputed;
};



}   // end namespace rms