// Author: Ryan Schmidt  (rms@unknownroad.com)  http://www.unknownroad.com
// Copyright (c) 2009. All Rights Reserved.
// libgeometry is supplied "AS IS". The Author disclaims all warranties, expressed or implied, including,
// without limitation, the warranties of merchantability and of fitness for any purpose. The Author assume no liability
// for direct, indirect, incidental, special, exemplary, or consequential damages, which may result from the use of
// libgeometry, even if advised of the possibility of such damage. Permission is hereby granted to use, copy, 
// modify, and distribute this source code, or portions hereof, for any purpose, without fee, subject to the following restrictions:
// 1) The origin of this source code must not be misrepresented.
// 2) This Copyright notice may not be removed or altered from any source or altered source distribution.
// The Author specifically permits, without fee, the use of this source code as a component in commercial products.
// [LICENSE]
#pragma once

#include "config.h"
#include <vector>
#include <VFTriangleMesh.h>

namespace rms {

class VertexWeights
{
public:

	/*
	 * weights that work for any neighbourhood
	 */

	static void Uniform( VFTriangleMesh & mesh, IMesh::VertexID vID, std::vector<IMesh::VertexID> & vNeighbourhood, 
						   std::vector<float> & vWeights, bool bNormalize = true );

	static void InverseDistance( VFTriangleMesh & mesh, IMesh::VertexID vID, std::vector<IMesh::VertexID> & vNeighbourhood, 
								 std::vector<float> & vWeights, float fPow = 2.0f, float fEps = 0.0001f, bool bNormalize = true );


	/*
	 * weights that only work for a vertex one-ring
	 */

	//! assumes one-ring is ordered
	static void Cotangent( VFTriangleMesh & mesh, IMesh::VertexID vID, std::vector<IMesh::VertexID> & vOneRing, 
						   std::vector<float> & vWeights, bool bNormalize = true );


};



}   // end namespace rms
